package com.cch_lab.android.ware.sample.mymessageapi;

import android.content.Context;
import android.os.AsyncTask;
import android.os.Bundle;
import android.util.Log;

import com.google.android.gms.common.api.GoogleApiClient;
import com.google.android.gms.wearable.MessageApi;
import com.google.android.gms.wearable.Node;
import com.google.android.gms.wearable.NodeApi;
import com.google.android.gms.wearable.Wearable;

import java.util.Collection;
import java.util.HashSet;

/**
 * MessageApi を利用した、スマートフォン ← → Wearのメッセージ送受信ユーティリティ．
 * <p>
 * 【参考元】
 * <ul>
 * <li>
 * Sending and Receiving Messages</br>
 * http://developer.android.com/training/wearables/data-layer/messages.html
 * </li>
 * <li>
 * ウェアラブル向けアプリを作る > メッセージの送信と受信</br>
 * http://firespeed.org/diary.php?diary=kenz-1760
 * </li>
 * <li>
 * Android Wearアプリケーション開発（2） > Android Wearのアプリの作り方 > Message APIによるデータ送受信</br>
 * http://www.buildinsider.net/mobile/androidwear/02
 * </li>
 * </ul>
 * </p>
 */
public class MyMessageApiUtil {
    private static final String TAG = "MyMessageApiUtil";
    private static final String MY_MESSAGE_API_UTIL_CLASS   = MyMessageApiUtil.class.getName();
    private static final String MY_MESSAGE_API_UTIL_NAME    = MyMessageApiUtil.class.getSimpleName();
    private static final String MY_MESSAGE_API_UTIL_PACKAGE = MY_MESSAGE_API_UTIL_CLASS.substring(0, MY_MESSAGE_API_UTIL_CLASS.length()- MY_MESSAGE_API_UTIL_NAME.length());

    /** MyMessageApiUtil・パス(厳密区別用)</br>
     * <ul><li>パスとは、フォワードスラッシュで始まる一意の文字列です。（例"/path/to/data"）</li></ul> */
    private static final String MY_MESSAGE_API_UTIL_STRICT_PATH = "/"+ MY_MESSAGE_API_UTIL_PACKAGE + "/" + MY_MESSAGE_API_UTIL_NAME;

    /** MyMessageApiUtil・パス(簡易用)</br>
     * <ul><li>パスとは、フォワードスラッシュで始まる一意の文字列です。（例"/path/to/data"）</li></ul> */
    private static final String MY_MESSAGE_API_UTIL_PATH = "/" + MY_MESSAGE_API_UTIL_NAME;


    /** MyMessageApiUtil シングルトン・インスタンス. */
    private static MyMessageApiUtil sInstance = new MyMessageApiUtil();

    /** コンテキスト. */
    private Context          mContext;

    /** Google API Client インスタンス. */
    private GoogleApiClient  mGoogleApiClient;

    /** 送受信対象メッセージ・パスのクエリ部配列. */
    private String[]         mTargetPathQueryArray;

    /** 受信対象メッセージリスナのアクション・インスタンス. */
    private MyWearableListenerService.MyWearableListenerAction mListenerAction;


    /**
     * My MessageApi Utility シングルトン・コンストラクタ．
     */
    private MyMessageApiUtil() {
    }

    /**
     * My MessageApi Utility 初期化．
     * <p>
     * スマートフォンとWear間のデータ送受信用の Google Play Service インスタンスを初期化します。<br/>
     * <strong>
     * Google Play Service インスタンスは、データ送受信する、スマートフォンとWear側の両者ともに生成する必要があります。
     * </strong>
     * </p>
     * @param context            コンテキスト
     * @param targetPathQueries 送受信対象メッセージ・パスのクエリ部(配列)
     * @param listenerAction    メッセージ受信リスナのアクション・インスタンス
     */
    public static void init(Context context,
                     String[] targetPathQueries,
                     MyWearableListenerService.MyWearableListenerAction listenerAction) {
        sInstance.initialize(context, targetPathQueries, listenerAction);
    }
    /**
     * My MessageApi Utility 初期化．
     * <p>
     * スマートフォンとWear間のデータ送受信用の Google Play Service インスタンスを初期化します。<br/>
     * <strong>
     * Google Play Service インスタンスは、データ送受信する、スマートフォンとWear側の両者ともに生成する必要があります。
     * </strong>
     * </p>
     * @param context            コンテキスト
     * @param targetPathQueries 送受信対象メッセージ・パスのクエリ部(配列)
     * @param listenerAction    メッセージ受信リスナのアクション・インスタンス
     */
    private void initialize(Context context,
                            String[] targetPathQueries,
                            MyWearableListenerService.MyWearableListenerAction listenerAction) {
        mContext              = context;
        mTargetPathQueryArray = targetPathQueries;
        mListenerAction       = listenerAction;

        //スマートフォンとWear間のデータ送受信用の Google Play Service を生成する。
        //（※）インスタンスは、データ送受信する、スマートフォンとWear側の両者ともに生成する必要があります。
        mGoogleApiClient = new GoogleApiClient.Builder(mContext)
                .addApi(Wearable.API)
                .addConnectionCallbacks(new GoogleApiClient.ConnectionCallbacks() {
                    @Override
                    public void onConnected(Bundle bundle) {
                        //接続(再接続)時の処理
                        Log.d(TAG, "Google Api Client connected");
                        //Toast.makeText(
                        //        mContext,
                        //        "MyMessageApiUtil\nGoogleApiClient connected",
                        //        Toast.LENGTH_LONG).show();
                    }

                    @Override
                    public void onConnectionSuspended(int i) {
                        //一時停止(一時切断)時の処理
                        Log.d(TAG, "Google Api Client suspended");
                        //Toast.makeText(
                        //        mContext,
                        //        "MyMessageApiUtil\nGoogleApiClient suspended",
                        //        Toast.LENGTH_LONG).show();
                    }
                }).build();
        mGoogleApiClient.connect();
    }


    /**
     * メッセージ送信．
     * <p>
     * MessageApi を利用した、スマートフォン → Wear
     * あるいは Wear → スマートフォン のメッセージ送信メソッドです。</     * メッセージ識別子としてパスに当該メッセージ用のクエリ(MY_MESSAGE_API_UTIL_PATH + sendPathQuery)を指定して
     * メッセージ送信先候補のノード（アカウント別接点端末）にメッセージを送信します。
     * </p>
     * @param sendPathQuery 当該送信メッセージのパス設定に追加するクエリ部
     * @param message  送信メッセージ
     */
    public static void sendMessage(final String sendPathQuery, final String message) {
            //Wearable.MessageApi.sendMessage() メソッドは、UIスレッドから実行すること。
            new AsyncTask<Void, Void, Void>() {
                @Override
                protected Void doInBackground(Void... params) {
                    sInstance.sendMessaging(sendPathQuery, message);
                    return null;
                }
            }.execute();
    }
    /**
     * メッセージ送信．
     * <p>
     * MessageApi を利用した、スマートフォン → Wear
     * あるいは Wear → スマートフォン のメッセージ送信メソッドです。</br>
     * メッセージ識別子としてパスに当該メッセージ用のクエリ(MY_MESSAGE_API_UTIL_PATH + sendPathQuery)を指定して
     * メッセージ送信先候補のノード（アカウント別接点端末）にメッセージを送信します。
     * </p>
     * @param sendPathQuery 当該送信メッセージのパス設定に追加するクエリ部
     * @param message  送信メッセージ
     */
    private void sendMessaging(String sendPathQuery, String message) {
        Log.d(TAG, "sendMessaging  start");
        if (mGoogleApiClient == null || mTargetPathQueryArray == null) {
            Log.d(TAG, "sendMessaging  end(null)");
            return;
        } else {
            boolean isHit = false;
            for (String targetQuery : mTargetPathQueryArray) {
                if (targetQuery.equals(sendPathQuery)){
                    isHit = true;
                    break;
                }
            }
            if (!isHit){
                Log.d(TAG, "sendMessaging  end(null)");
                return;
            }
        }

        Collection<String> nodes = getNodes();
        for (String node : nodes) {
            MessageApi.SendMessageResult result =
                    Wearable.MessageApi.sendMessage(
                            mGoogleApiClient,
                            node,
                            createMessagePath(sendPathQuery),
                            message.getBytes()).await();
            if (!result.getStatus().isSuccess()) {
                Log.e(TAG, "ERROR: failed to send Message: " + result.getStatus());
            } else {
                Log.d(TAG, "sendMessaging  sendPathQuery=" + sendPathQuery + ", message=" + message);
            }
        }
        Log.d(TAG, "sendMessaging  end");
    }

    /**
     * メッセージパス作成．
     * <p>
     * メッセージ識別用のクエリ(MY_MESSAGE_API_UTIL_PATH + query)を付加した
     * メッセージ・パスを作成します。
     * </p>
     * @param query メッセージ識別用パスのクエリ部
     * @return 生成メッセージ・パス
     */
    public static String createMessagePath(String query) {
        return MY_MESSAGE_API_UTIL_PATH + "/" + query;
    }

    /**
     * メッセージ送信先候補のノード・コレクション取得．
     * <p>
     * メッセージ送信先候補のノード（アカウント別接点端末）・コレクションを取得します。
     * </p>
     * @return メッセージ送信先候補ノード・コレクション (※)null が返ることもあります。
     */
    private Collection<String> getNodes() {
        if (mGoogleApiClient == null) {
            return null;
        }

        HashSet<String> results = new HashSet<String>();
        NodeApi.GetConnectedNodesResult nodes =
                Wearable.NodeApi.getConnectedNodes(mGoogleApiClient).await();
        int index = 0;
        for (Node node : nodes.getNodes()) {
            results.add(node.getId());
            Log.d(TAG, "getNodes [" + index + "]"
                    + "  ID=" + node.getId()
                    + ", displayName=" + node.getDisplayName());
        }
       return results;
    }

    /**
     * 送受信対象メッセージ・パスのクエリ部．
     * <p>
     * 当該ユーティリティ初期化メソッドで設定した、
     * 当該ユーティリティのメッセージ送受信で想定している、
     * 対象送受信メッセージ・パスのクエリ部配列を取得します。
     * </p>
     * @return 送受信対象メッセージ・パスのクエリ部配列 (※)null が返ることもあります。
     */
    public static String[] getsReceivePathQueryArray() {
        return sInstance.mTargetPathQueryArray;
    }

    /**
     * メッセージ受信リスナのアクション・インスタンス取得．
     * <p>
     * 当該ユーティリティ初期化メソッドで設定した、
     * 当該ユーティリティのメッセージ受信で想定している、
     * メッセージ受信リスナのアクション・インスタンスを取得します。
     * </p>
     * @return メッセージ受信リスナのアクション・インスタンス (※)null が返ることもあります。
     */
    public static MyWearableListenerService.MyWearableListenerAction getListenerAction() {
        return sInstance.mListenerAction;
    }
}